/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

/*
 * Abstract:
 *	Declaration of thread pool for parallel computing.
 */

#ifndef _CL_MP_THREAD_POOL_H_
#define _CL_MP_THREAD_POOL_H_

#include <pthread.h>
#include <complib/cl_types.h>
#include <complib/cl_threadpool.h>

#ifdef __cplusplus
#  define BEGIN_C_DECLS extern "C" {
#  define END_C_DECLS   }
#else				/* !__cplusplus */
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif				/* __cplusplus */

BEGIN_C_DECLS
/****h* Component Library/Thread Pool
* NAME
*	Thread Pool for multi purpose Parallel computing
*
* DESCRIPTION
*	The Thread Pool for multi purpose Parallel Computing manages a user specified number
*	of threads allocated from regular thread pool
*
*	Each thread in the thread pool is placed on separate processor to speed
*	up processing. Each time the user provides a function and a collection.
*	All threads in the MP thread pool invoke these
*	callback function on all the members of the collection and performs parallel computations.
*
*	The thread pool functions operate on a cl_pc_thread_pool_t structure which
*	should be treated as opaque, and should be manipulated only through the
*	provided functions.
*
* SEE ALSO
*	Structures:
*		cl_MP_thread_pool_t
*
*	Initialization:
*		cl_mp_thread_pool_init, cl_mp_thread_pool_destroy
*
*	Manipulation
*		cl_mp_thread_pool_array_apply
*********/

/****s* Component Library: MP Thread Pool/cl_mp_thread_pool_t
* NAME
*	cl_mp_thread_pool_t
*
* DESCRIPTION
*	Thread pool structure for multi purpose use.
*	After one init can be used multiple times for Parallel computing.
*
*	The cl_mp_thread_pool_t structure should be treated as opaque, and should be
*	manipulated only through the provided functions.
*
* SYNOPSIS
*/
typedef struct cl_mp_thread_pool {
	cl_thread_pool_t thread_pool;
	void (*pfn_callback) (void *, void *);
	void *contex;
	void **task_queue;
	int queue_len;
	int num_pending_tasks;
	pthread_mutex_t mutex;
	pthread_cond_t cond;
	uint16_t *active_cores;
	uint16_t max_threads_per_core;
} cl_mp_thread_pool_t;
/*
* FIELDS
*	thread_pool
*		Thread pool struct.
*
*	pfn_callback
*		Thread callback function for parallel computing
*
*	context
*		Context to pass to the thread callback function.
*
*	task_queue
*		A queue of tasks for the threads to complete.
*
*	queue_len
*		Number of tasks in the task_queue.
*
*	mutex
*		Mutex for threadpool use.
*
*	cond
*		Conditional variable to signal an event to thread
*
*	active_cores
*		Array indicating current number of active threads per core 
*
*	max_threads_per_core
*		Indicates maximal number of threads allowed
*		to be placed on separate processor
*
* SEE ALSO
*	Thread Pool
*********/

void cl_mp_thread_pool_array_apply(IN cl_mp_thread_pool_t * const p_mp_thread_pool,
				   IN void * tasks[],
				   IN unsigned num_tasks,
				   IN void (*pfn_callback) (void *, void *),
				   IN void *contex);

cl_status_t cl_mp_thread_pool_init(IN cl_mp_thread_pool_t * const p_mp_thread_pool,
				   IN unsigned num_threads,
				   IN uint16_t max_threads_per_core,
				   IN const char *const name);

void cl_mp_thread_pool_destroy(IN cl_mp_thread_pool_t * const p_mp_thread_pool);

END_C_DECLS
#endif				/* _CL_MP_THREAD_POOL_H_ */
